
#include <stdio.h>
#include <helianthus.h>


const char grayscaleChars[5][10] = { " ", "░", "▒", "▓", "█" };

int conType = 0;
char conAddress[1000] = "";

int width;
int height;
unsigned char *pixels;


void generateColoredBB(FILE *f, int width, int height, unsigned char *pixels) {
  for(int i = 0; i < height; i++) {
    for(int j = 0; j < width; j++) {
      unsigned int pixel = imageGetPixel(width, height, pixels, j, i);
      fprintf(f, "[color=#%06X]██[/color]", pixel/256u);
    }
    fprintf(f, "\n");
  }
}


void generateColoredHTML(FILE *f, int width, int height, unsigned char *pixels) {
  for(int i = 0; i < height; i++) {
    for(int j = 0; j < width; j++) {
      unsigned int pixel = imageGetPixel(width, height, pixels, j, i);
      fprintf(f, "<span style=\"color: #%06X;\">██</span>", pixel/256u);
    }
    fprintf(f, "<br />\n");
  }
}


void generateGrayscale(FILE *f, int width, int height, unsigned char *pixels) {
  for(int i = 0; i < height; i++) {
    for(int j = 0; j < width; j++) {
      unsigned int pixel = imageGetPixel(width, height, pixels, j, i);
      double brightness = (colorGetRed(pixel) + colorGetGreen(pixel) + colorGetBlue(pixel))/3.0;
      int index = (int)(brightness*4.999);
      fprintf(f, "%s", grayscaleChars[4 - index]);
      fprintf(f, "%s", grayscaleChars[4 - index]);
    }
    fprintf(f, "\n");
  }
}



void init() {
    messageBox(
      "You need to place images into the folder \"images\".\n\n"
      "Address example: images/image.png\n\n\n\n"
      "The output may differs from the image thumbnail!" );
    askTextf("What conversion type number? (1 - HTML, 2 - BBCode, 3 - Gray scale)", "%d", &conType);
    askText("What image address?", conAddress, 1000);
    if (imageLoad(conAddress, &width, &height, &pixels)) {
      FILE *f = fopen("text.txt", "w");
      if (f) {
        switch(conType) {
        case 2:  generateColoredBB  (f, width, height, pixels); break;
        case 3:  generateGrayscale  (f, width, height, pixels); break;
        default: generateColoredHTML(f, width, height, pixels); break;
        }
        fclose(f);
      } else {
        messageBox("Error: Cannot open file for write");
      }
    } else {
      messageBox("Error: Cannot load image");
    }
}

void draw() {
  saveState();
  zoom(10);
  textSize(1);
  noFill();
  for(int i = 0; i < height; i++) {
    for(int j = 0; j < width; j++) {
      stroke(imageGetPixel(width, height, pixels, j, i));
      text(j, i, "#");
    }
  }
  restoreState();
}

int main() {
    windowSetInit(&init);
    windowSetDraw(&draw);
    windowRun();
    return 0;
}
